/* #Specification: FILE_CFG / principles
	The FILE_CFG object is indeed a FILE object (from stdio.h).
	Our goal is to overload the fclose stdio function so we know
	when a CONFIG_FILE is updated. The function CONFIG_FILE::fopen return
	a FILE_CFG pointer.

	To overload fclose, we have to overload few things as well: fputs, fprintf
	, etc ...).

	Now, FILE_CFG could be a little smarter. It could be a real object
	containing a FILE pointer and few other things, why not. By using
	this trickery, we can overload fclose and yet, keep some binary
	compatibility with old modules out there. For sure, those old
	module won't use the new fclose. Kind of a transition strategy.
*/
#include <stdio.h>
#include <stdarg.h>
#include "misc.h"

/*
	Replacement for stdio function using FILE_CFG
*/
int fclose (FILE_CFG *fout)
{
	return fclose ((FILE*)fout);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fputs (const char *s, FILE_CFG *fout)
{
	return fputs (s,(FILE*)fout);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fputc (char c, FILE_CFG *fout)
{
	return fputc (c,(FILE*)fout);
}


/*
	Replacement for stdio function using FILE_CFG
*/
char *fgets (char *s, int size, FILE_CFG *fout)
{
	return fgets (s,size,(FILE*)fout);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fprintf (FILE_CFG *fout, const char *ctl, ...)
{
	va_list list;
	va_start (list,ctl);
	int ret = vfprintf ((FILE*)fout,ctl,list);
	va_end (list);
	return ret;
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fwrite (const void *ptr, size_t size, size_t n, FILE_CFG *fout)
{
	return fwrite (ptr,size,n,(FILE*)fout);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fread (void *ptr, size_t size, size_t n, FILE_CFG *fout)
{
	return fread (ptr,size,n,(FILE*)fout);
}

void comment_write (const SSTRING &str, FILE_CFG *fout, char comchar)
{
	comment_write (str,(FILE*)fout,comchar);
}

void comment_write (const SSTRING &str, FILE_CFG *fout)
{
	comment_write (str,(FILE*)fout);
}
const char *fgets_comments (
	char buf[],
	int size,
	FILE_CFG *fin,
	SSTRING &comments,
	char comchar)
{
	return fgets_comments (buf,size,(FILE*)fin,comments,comchar);
}

const char *fgets_comments (
	char buf[],
	int size,
	FILE_CFG *fin,
	SSTRING &comments)
{
	return fgets_comments (buf,size,(FILE*)fin,comments);
}

char *fgets_strip (
	char *buf,
	int sizebuf,
	FILE_CFG *fin,
	char contcar,
	char comcar,
	int *noline)
{
	return fgets_strip(buf,sizebuf,(FILE*)fin,contcar,comcar,noline);
}

char *fgets_strip (
	char *buf,
	int sizebuf,
	FILE_CFG *fin,
	int *noline)
{
	return fgets_strip(buf,sizebuf,(FILE*)fin,noline);
}

int fgets_cont (char *buf, int size, FILE_CFG *fin, bool cont)
{
	return fgets_cont (buf,size,(FILE*)fin,cont);
}

int fgets_cont (char *buf, int size, FILE_CFG *fin)
{
	return fgets_cont (buf,size,(FILE*)fin);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fflush (FILE_CFG *fout)
{
	return fflush((FILE*)fout);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fscanf (FILE_CFG *fin, const char *ctl, ...)
{
	va_list list;
	va_start (list,ctl);
	int ret = vfscanf ((FILE*)fin,ctl,list);
	va_end (list);
	return ret;
}

/*
	Replacement for stdio function using FILE_CFG
*/
void rewind (FILE_CFG *f)
{
	rewind ((FILE*)f);
}

/*
	Replacement for stdio function using FILE_CFG
*/
long ftell (FILE_CFG *f)
{
	return ftell ((FILE*)f);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fseek (FILE_CFG *f, long offset, int whence)
{
	return fseek ((FILE*)f,offset,whence);
}

/*
	Replacement for stdio function using FILE_CFG
*/
FILE_CFG *fopen_cfg (const char *fname, const char *mode)
{
	return (FILE_CFG*)fopen(fname,mode);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int feof (FILE_CFG *f)
{
	return feof((FILE*)f);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fgetc (FILE_CFG *f)
{
	return getc((FILE*)f);
}
/*
	Replacement for stdio function using FILE_CFG
*/
int fputc (int c, FILE_CFG *f)
{
	return putc (c,(FILE*)f);
}

/*
	Replacement for stdio function using FILE_CFG
*/
int fileno (FILE_CFG *f)
{
	return fileno((FILE*)f);
}

